

(* ---------- integers as module item --- *)

SPECIFICATION NINT;
(*Non-negative integer specification. *)
(*1*) IMPORT AGROUP [ nint/ag, 
                      IREAD/READ, IWRITE/WRITE, 
                      Izero/ZERO, ISUM/SUM, IDIF/DIF, INEG/NEG ];
(*9*) END NINT.


MODEL AGROUP;
(*The additve group of integers is a model for abelian group. *)
(*1*) IMPORT NINT;
(*2*) MAP READ(nint)       -> IREAD(); 
      MAP WRITE(nint)      -> IWRITE(VAL);
(*3*) MAP ZERO(nint)       -> Izero();
(*4*) MAP SUM(nint,nint)   -> ISUM(VAL,VAL);
      MAP DIF(nint,nint)   -> IDIF(VAL,VAL);
      MAP NEG(nint)        -> INEG(VAL);
(*9*) END AGROUP.


(* ---------- free module item --- *)

SPECIFICATION FMODULE(Ring (*:spec*), ring,set (*: sorts*));
(*Free module specification as concretization of a module. *)
(*1*) IMPORT MODULE(Ring) [ fm/melem, 
                            FWRITE/WRITE, FREAD/READ,
                            FZERO/ZERO, FSUM/SUM, FDIF/DIF, FNEG/NEG, 
                            FSPROD/SPROD ];
      SIGNATURE monom(ring,set): fm;
(*9*) END FMODULE.


IMPLEMENTATION FMODULE(Ring (*:spec*), ring,set (*: sorts*));
(*Module implementation. *)
      IMPORT REPRESENTATION[ set/name, ring/elem, fm/rep ];
      IMPORT RING;
      IMPORT Ring;
      VAR   zero: ring; 

(*1*) PROCEDURE FZERO(): fm;      
      BEGIN RETURN(NewRep());
      END FZERO;

(*2*) PROCEDURE monom(n,e): fm;      
      VAR   m: fm;
      BEGIN m:=NewRep(); SetRep(e,n,m);
            RETURN(m);
      END monom;

(*3*) PROCEDURE FSUM(t1,t2): fm;      
            VAR   x1, x2:  fm;
            PROCEDURE sum(n,e): pair;
            VAR   ep: ring; 
            BEGIN ep:=GetRep(n,x1);
                  IF ep <> NullPair() THEN e:=SUM(ep,e) END;
                  SetRep(n,e,x1);
                  RETURN(NullPair()) END sum;
            PROCEDURE filter(n,e): pair;
            BEGIN IF EQUAL(e,zero) = 1 THEN RETURN(NullPair()) END;
                  RETURN(Pair(n,e)) END filter;
      BEGIN x1:=CopyRep(t1); 
            x2:=ForEachinRep(t2,sum); (* side effect on x1 *)
            x1:=ForEachinRep(x1,filter); (*remove zeros *)
            RETURN(x1);
      END FSUM;

(*4*) PROCEDURE FNEG(t): fm;      
            PROCEDURE neg(n,e): pair;
            RETURN(Pair(n,NEG(e))) neg;
      BEGIN RETURN(ForEachinRep(t,neg));
      END FNEG;

(*5*) PROCEDURE FDIF(t1,t2): fm;      
      BEGIN RETURN(FSUM(t1,FNEG(t2)));
      END FDIF;

(*6*) PROCEDURE FSPROD(s,t): fm;      
            PROCEDURE prod(n,e): pair;
            VAR   ep: ring;
            BEGIN ep:=PROD(s,e);
                  IF EQUAL(ep,zero) = 1 THEN RETURN(NullPair()) END;
                  RETURN(Pair(n,ep)) END prod;
      BEGIN RETURN(ForEachinRep(t,prod)); 
      END FSPROD;

(*4*) PROCEDURE FWRITE(t);      
            PROCEDURE wrt(n,e): pair;
            BEGIN WRITE(e); CLOUT(" "); WRITE(n); CLOUT(", ");  
                  RETURN(NullPair()) END wrt;
      BEGIN ForEachinRep(t,wrt); RETURN(NullPair());
      END FWRITE;

      BEGIN 
            zero:=ZERO(ring);
(*9*) END FMODULE.


MODEL MODULE(Ring);
(*A free module is a model for modules. *)
(*1*) IMPORT FMODULE(INTEGER,INT,INT);
(*2*) MAP WRITE(fm)      -> FWRITE(VAL);
(*3*) MAP ZERO(fm)       -> FZERO();
(*4*) MAP SUM(fm,fm)     -> FSUM(VAL,VAL);
      MAP DIF(fm,fm)     -> FDIF(VAL,VAL);
      MAP NEG(fm)        -> FNEG(VAL);
(*5*) MAP SPROD(ring,fm) -> FSPROD(VAL,VAL);
(*9*) END MODULE.


